@@ -2,11 +2,11 @@
 **  Copyright (c) 2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dkim-ar.c,v 1.7 2007/09/07 00:28:57 msk Exp $
+**  $Id: dkim-ar.c,v 1.16 2007/12/18 22:44:13 msk Exp $
 */
 
 #ifndef lint
-static char dkim_ar_c_id[] = "@(#)$Id: dkim-ar.c,v 1.7 2007/09/07 00:28:57 msk Exp $";
+static char dkim_ar_c_id[] = "@(#)$Id: dkim-ar.c,v 1.16 2007/12/18 22:44:13 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -15,12 +15,16 @@ static char dkim_ar_c_id[] = "@(#)$Id: dkim-ar.c,v 1.7 2007/09/07 00:28:57 msk E
 #include <ctype.h>
 #include <assert.h>
 #include <string.h>
+#ifdef ARTEST
+# include <sysexits.h>
+#endif /* ARTEST */
 
 /* dkim-filter includes */
 #include "dkim-ar.h"
 
 /* macros */
-#define	ARES_ENDOF(x)	((x) + sizeof(x) - 1)
+#define	ARES_ENDOF(x)		((x) + sizeof(x) - 1)
+#define	ARES_STRORNULL(x)	((x) == NULL ? "(null)" : (x))
 
 /* states */
 #define AUTHRES_STATE_UNKNOWN	(-1)
@@ -30,6 +34,7 @@ static char dkim_ar_c_id[] = "@(#)$Id: dkim-ar.c,v 1.7 2007/09/07 00:28:57 msk E
 #define AUTHRES_STATE_PTYPE	3
 #define AUTHRES_STATE_PROPERTY	4
 #define AUTHRES_STATE_VALUE	5
+#define AUTHRES_STATE_POSTVALUE	6
 
 /* tables */
 struct lookup
@@ -42,7 +47,9 @@ struct lookup methods[] =
 {
 	{ "auth",		ARES_METHOD_AUTH },
 	{ "dkim",		ARES_METHOD_DKIM },
+	{ "dkim-ssp",		ARES_METHOD_DKIMSSP },
 	{ "domainkeys",		ARES_METHOD_DOMAINKEYS },
+	{ "iprev",		ARES_METHOD_IPREV },
 	{ "senderid",		ARES_METHOD_SENDERID },
 	{ "spf",		ARES_METHOD_SPF },
 	{ NULL,			ARES_METHOD_UNKNOWN }
@@ -133,6 +140,36 @@ ares_convert(struct lookup *table, char *str)
 	/* NOTREACHED */
 }
 
+#ifdef ARTEST
+/*
+**  ARES_XCONVERT -- convert a code to its string
+**
+**  Parameters:
+**  	table -- in which table to look up
+**  	code -- code to find
+**
+**  Return value:
+**  	A string translation of "code".
+*/
+
+static char *
+ares_xconvert(struct lookup *table, int code)
+{
+	int c;
+
+	assert(table != NULL);
+
+	for (c = 0; ; c++)
+	{
+		if (table[c].str == NULL || table[c].code == code)
+			return table[c].str;
+	}
+
+	/* NOTREACHED */
+}
+
+#endif /* ARTEST */
+
 /*
 **  AUTHRES_PARSE -- parse an Authentication-Results: header, return a
 **                   structure containing a parsed result
@@ -152,6 +189,7 @@ ares_parse(u_char *hdr, struct authres *ar)
 	bool escaped;
 	int paren;
 	int n;
+	int c;
 	int state;
 	u_char *p;
 	u_char *q;
@@ -228,6 +266,7 @@ ares_parse(u_char *hdr, struct authres *ar)
 
 	escaped = FALSE;
 	n = 0;
+	c = 0;
 
 	/* now break it into its component pieces */
 	for (p = tmp; *p != '\0'; p++)
@@ -235,7 +274,7 @@ ares_parse(u_char *hdr, struct authres *ar)
 		switch (state)
 		{
 		  case AUTHRES_STATE_HOST:		/* hostname */
-			if (*p == ';' && !escaped)
+			if (*p == ';')
 			{
 				ares_trimspaces(ar->ares_host);
 				state = AUTHRES_STATE_METHOD;
@@ -250,7 +289,7 @@ ares_parse(u_char *hdr, struct authres *ar)
 			if (ar->ares_host[0] != '\0' ||
 			    !(isascii(*p) && isspace(*p)))
 			{
-				if (q <= end)
+				if (q < end)
 				{
 					*q = *p;
 					q++;
@@ -260,18 +299,19 @@ ares_parse(u_char *hdr, struct authres *ar)
 			break;
 
 		  case AUTHRES_STATE_METHOD:
-			if (*p == '=' && !escaped)
+			if (*p == '=')
 			{
 				ares_trimspaces(tmp2);
 
 				n++;
+				c = 0;
 
 				/* too many! */
 				if (n >= MAXARESULTS)
 					return -1;
 
-				ar->ares_method[n - 1] = ares_convert(methods,
-				                                      tmp2);
+				ar->ares_result[n - 1].result_method = ares_convert(methods,
+				                                                    tmp2);
 
 				state = AUTHRES_STATE_RESULT;
 
@@ -283,7 +323,7 @@ ares_parse(u_char *hdr, struct authres *ar)
 
 			if (tmp2[0] != '\0' || !(isascii(*p) && isspace(*p)))
 			{
-				if (q <= end)
+				if (q < end)
 				{
 					*q = *p;
 					q++;
@@ -293,12 +333,12 @@ ares_parse(u_char *hdr, struct authres *ar)
 			break;
 
 		  case AUTHRES_STATE_RESULT:
-			if (isascii(*p) && isspace(*p) && !escaped)
+			if (isascii(*p) && isspace(*p))
 			{
 				ares_trimspaces(tmp2);
 
-				ar->ares_result[n - 1] = ares_convert(aresults,
-				                                      tmp2);
+				ar->ares_result[n - 1].result_result = ares_convert(aresults,
+				                                                    tmp2);
 
 				state = AUTHRES_STATE_PTYPE;
 
@@ -310,7 +350,7 @@ ares_parse(u_char *hdr, struct authres *ar)
 
 			if (tmp2[0] != '\0' || !(isascii(*p) && isspace(*p)))
 			{
-				if (q <= end)
+				if (q < end)
 				{
 					*q = *p;
 					q++;
@@ -320,25 +360,34 @@ ares_parse(u_char *hdr, struct authres *ar)
 			break;
 
 		  case AUTHRES_STATE_PTYPE:
-			if (*p == '.' && !escaped)
+			if (*p == '.')
 			{
 				ares_trimspaces(tmp2);
 
-				ar->ares_ptype[n - 1] = ares_convert(ptypes,
-				                                     tmp2);
+				c++;
+
+				/* too many! */
+				if (c >= MAXPROPS)
+					return -1;
+
+				ar->ares_result[n - 1].result_props = c;
+
+
+				ar->ares_result[n - 1].result_ptype[c - 1] = ares_convert(ptypes,
+				                                                          tmp2);
 
 				state = AUTHRES_STATE_PROPERTY;
 
-				q = ar->ares_property[n - 1];
-				end = ARES_ENDOF(ar->ares_property[n - 1]);
+				q = ar->ares_result[n - 1].result_property[c - 1];
+				end = ARES_ENDOF(ar->ares_result[n - 1].result_property[c - 1]);
 
 				break;
 			}
 
-			if (ar->ares_property[n - 1][0] != '\0' ||
+			if (ar->ares_result[n - 1].result_property[c - 1][0] != '\0' ||
 			    !(isascii(*p) && isspace(*p)))
 			{
-				if (q <= end)
+				if (q < end)
 				{
 					*q = *p;
 					q++;
@@ -348,24 +397,24 @@ ares_parse(u_char *hdr, struct authres *ar)
 			break;
 
 		  case AUTHRES_STATE_PROPERTY:
-			if (*p == '=' && !escaped)
+			if (*p == '=')
 			{
-				ares_trimspaces(ar->ares_property[n - 1]);
+				ares_trimspaces(ar->ares_result[n - 1].result_property[c - 1]);
 
 				/* XXX -- convert */
 
 				state = AUTHRES_STATE_VALUE;
 
-				q = ar->ares_value[n - 1];
-				end = ARES_ENDOF(ar->ares_value[n - 1]);
+				q = ar->ares_result[n - 1].result_value[c - 1];
+				end = ARES_ENDOF(ar->ares_result[n - 1].result_value[c - 1]);
 
 				break;
 			}
 
-			if (ar->ares_value[n - 1][0] != '\0' ||
+			if (ar->ares_result[n - 1].result_property[c - 1][0] != '\0' ||
 			    !(isascii(*p) && isspace(*p)))
 			{
-				if (q <= end)
+				if (q < end)
 				{
 					*q = *p;
 					q++;
@@ -375,12 +424,26 @@ ares_parse(u_char *hdr, struct authres *ar)
 			break;
 
 		  case AUTHRES_STATE_VALUE:
-			if (*p == ';' && !escaped)
+			if (*p == ';')
 			{
-				ares_trimspaces(ar->ares_value[n - 1]);
+				ares_trimspaces(ar->ares_result[n - 1].result_value[c - 1]);
 
 				state = AUTHRES_STATE_METHOD;
 
+				ar->ares_result[n - 1].result_props = c;
+
+				q = tmp2;
+				end = tmp2 + sizeof tmp2 - 1;
+				memset(tmp2, '\0', sizeof tmp2);
+
+				break;
+			}
+			else if (isascii(*p) && isspace(*p))
+			{
+				ares_trimspaces(ar->ares_result[n - 1].result_value[c - 1]);
+
+				state = AUTHRES_STATE_POSTVALUE;
+
 				q = tmp2;
 				end = tmp2 + sizeof tmp2 - 1;
 				memset(tmp2, '\0', sizeof tmp2);
@@ -388,10 +451,10 @@ ares_parse(u_char *hdr, struct authres *ar)
 				break;
 			}
 
-			if (ar->ares_value[n - 1][0] != '\0' ||
+			if (ar->ares_result[n - 1].result_value[c - 1][0] != '\0' ||
 			    !(isascii(*p) && isspace(*p)))
 			{
-				if (q <= end)
+				if (q < end)
 				{
 					*q = *p;
 					q++;
@@ -400,16 +463,112 @@ ares_parse(u_char *hdr, struct authres *ar)
 
 			break;
 
+		  case AUTHRES_STATE_POSTVALUE:
+			if (*p == ';')
+			{
+				state = AUTHRES_STATE_METHOD;
+			}
+			else if (!(isascii(*p) && isspace(*p)))
+			{
+				state = AUTHRES_STATE_PTYPE;
+
+				q = tmp2;
+				end = tmp2 + sizeof tmp2 - 1;
+				memset(tmp2, '\0', sizeof tmp2);
+				*q++ = *p;
+			}
+			break;
+
 		  default:
 			return -1;
 		}
 	}
 
 	/* end-of-string may mean there was data to process */
-	if (state == AUTHRES_STATE_VALUE && ar->ares_value[0] != '\0')
-		ares_trimspaces(ar->ares_value[n - 1]);
+	if (state == AUTHRES_STATE_VALUE &&
+	    ar->ares_result[n - 1].result_value[c - 1][0] != '\0')
+	{
+		ares_trimspaces(ar->ares_result[n - 1].result_value[c - 1]);
+	}
+	else if (state == AUTHRES_STATE_RESULT && tmp2[0] != '\0')
+	{
+		ares_trimspaces(tmp2);
+		ar->ares_result[n - 1].result_result = ares_convert(aresults,
+		                                                    tmp2);
+	}
+	else
+	{
+		return -1;
+	}
 
 	ar->ares_count = n;
 
 	return 0;
 }
+
+#ifdef ARTEST
+/*
+**  MAIN -- program mainline
+**
+**  Parameters:
+**  	argc, argv -- the usual
+**
+**  Return value:
+**  	EX_USAGE or EX_OK
+*/
+
+int
+main(int argc, char **argv)
+{
+	int c;
+	int d;
+	int status;
+	char *p;
+	char *progname;
+	struct authres ar;
+
+	progname = (p = strrchr(argv[0], '/')) == NULL ? argv[0] : p + 1;
+
+	if (argc != 2)
+	{
+		printf("%s: usage: %s header-value\n", progname, progname);
+		return EX_USAGE;
+	}
+
+	status = ares_parse(argv[1], &ar);
+	if (status == -1)
+	{
+		printf("%s: ares_parse() returned -1\n", progname);
+		return EX_OK;
+	}
+
+	printf("%d result%s found\n", ar.ares_count,
+	       ar.ares_count == 1 ? "" : "s");
+
+	for (c = 0; c < ar.ares_count; c++)
+	{
+		printf("result #%d, %d propert%s\n", c,
+		       ar.ares_result[c].result_props,
+		       ar.ares_result[c].result_props == 1 ? "y" : "ies");
+
+		printf("\tmethod \"%s\"\n",
+		       ares_xconvert(methods,
+		                     ar.ares_result[c].result_method));
+		printf("\tresult \"%s\"\n",
+		       ares_xconvert(aresults,
+		                     ar.ares_result[c].result_result));
+
+		for (d = 0; d < ar.ares_result[c].result_props; d++)
+		{
+			printf("\tproperty #%d\n", d);
+			printf("\t\tptype \"%s\"\n",
+			       ares_xconvert(ptypes,
+			                     ar.ares_result[c].result_ptype[d]));
+			printf("\t\tproperty \"%s\"\n",
+			       ar.ares_result[c].result_property[d]);
+			printf("\t\tvalue \"%s\"\n",
+			       ar.ares_result[c].result_value[d]);
+		}
+	}
+}
+#endif /* ARTEST */
