@@ -2,14 +2,14 @@
 **  Copyright (c) 2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dkim-ar.h,v 1.4 2007/09/07 00:28:57 msk Exp $
+**  $Id: dkim-ar.h,v 1.6 2007/12/18 22:42:16 msk Exp $
 */
 
 #ifndef _DKIM_AR_H_
 #define _DKIM_AR_H_
 
 #ifndef lint
-static char dkim_ar_h_id[] = "@(#)$Id: dkim-ar.h,v 1.4 2007/09/07 00:28:57 msk Exp $";
+static char dkim_ar_h_id[] = "@(#)$Id: dkim-ar.h,v 1.6 2007/12/18 22:42:16 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -20,6 +20,7 @@ static char dkim_ar_h_id[] = "@(#)$Id: dkim-ar.h,v 1.4 2007/09/07 00:28:57 msk E
 
 /* limits */
 #define	MAXARESULTS	16
+#define	MAXPROPS	16
 #define	MAXAVALUE	256
 
 /* ARES_METHOD_T -- type for specifying an authentication method */
@@ -31,6 +32,8 @@ typedef int ares_method_t;
 #define	ARES_METHOD_DOMAINKEYS	2
 #define	ARES_METHOD_SENDERID	3
 #define	ARES_METHOD_SPF		4
+#define	ARES_METHOD_DKIMSSP	5
+#define	ARES_METHOD_IPREV	6
 
 /* ARES_RESULT_T -- type for specifying an authentication result */
 typedef int ares_result_t;
@@ -52,16 +55,23 @@ typedef int ares_ptype_t;
 #define	ARES_PTYPE_BODY		2
 #define	ARES_PTYPE_POLICY	3
 
-/* AUTHRES structure */
+/* RESULT structure -- a single result */
+struct result
+{
+	int		result_props;
+	ares_method_t	result_method;
+	ares_result_t	result_result;
+	ares_ptype_t	result_ptype[MAXPROPS];
+	char		result_property[MAXPROPS][MAXAVALUE + 1];
+	char		result_value[MAXPROPS][MAXAVALUE + 1];
+};
+
+/* AUTHRES structure -- the entire header parsed */
 struct authres
 {
 	int		ares_count;
-	ares_method_t	ares_method[MAXARESULTS];
-	ares_result_t	ares_result[MAXARESULTS];
-	ares_ptype_t	ares_ptype[MAXARESULTS];
 	char		ares_host[DKIM_MAXHOSTNAMELEN + 1];
-	char		ares_property[MAXARESULTS][MAXAVALUE + 1];
-	char		ares_value[MAXARESULTS][MAXAVALUE + 1];
+	struct result	ares_result[MAXARESULTS];
 };
 
 /*
