@@ -2,14 +2,14 @@
 **  Copyright (c) 2006-2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dkim-config.h,v 1.37 2007/10/24 06:23:47 msk Exp $
+**  $Id: dkim-config.h,v 1.40 2007/12/13 23:07:22 msk Exp $
 */
 
 #ifndef _DKIM_CONFIG_H_
 #define _DKIM_CONFIG_H_
 
 #ifndef lint
-static char dkim_config_h_id[] = "@(#)$Id: dkim-config.h,v 1.37 2007/10/24 06:23:47 msk Exp $";
+static char dkim_config_h_id[] = "@(#)$Id: dkim-config.h,v 1.40 2007/12/13 23:07:22 msk Exp $";
 #endif /* !lint */
 
 struct configdef dkimf_config[] =
@@ -30,6 +30,7 @@ struct configdef dkimf_config[] =
 	{ "KeyFile",			CONFIG_TYPE_STRING,	FALSE },
 	{ "KeyList",			CONFIG_TYPE_STRING,	FALSE },
 	{ "InternalHosts",		CONFIG_TYPE_STRING,	FALSE },
+	{ "MaximumHeaders",		CONFIG_TYPE_INTEGER,	FALSE },
 	{ "MaximumSignedBytes",		CONFIG_TYPE_INTEGER,	FALSE },
 	{ "MacroList",			CONFIG_TYPE_STRING,	FALSE },
 	{ "Minimum",			CONFIG_TYPE_STRING,	FALSE },
@@ -37,9 +38,11 @@ struct configdef dkimf_config[] =
 	{ "MTA",			CONFIG_TYPE_STRING,	FALSE },
 	{ "OmitHeaders",		CONFIG_TYPE_STRING,	FALSE },
 	{ "On-BadSignature",		CONFIG_TYPE_STRING,	FALSE },
+	{ "On-Default",			CONFIG_TYPE_STRING,	FALSE },
 	{ "On-DNSError",		CONFIG_TYPE_STRING,	FALSE },
 	{ "On-InternalError",		CONFIG_TYPE_STRING,	FALSE },
 	{ "On-NoSignature",		CONFIG_TYPE_STRING,	FALSE },
+	{ "On-Security",		CONFIG_TYPE_STRING,	FALSE },
 	{ "On-SignatureMissing",	CONFIG_TYPE_STRING,	FALSE },
 	{ "PeerList",			CONFIG_TYPE_STRING,	FALSE },
 	{ "PidFile",			CONFIG_TYPE_STRING,	FALSE },
