@@ -1,19 +1,19 @@
 /*
-**  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2005-2008 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dkim-filter.h,v 1.134 2007/11/30 19:42:59 msk Exp $
+**  $Id: dkim-filter.h,v 1.149 2008/01/25 18:45:56 msk Exp $
 */
 
 #ifndef _DKIM_FILTER_H_
 #define _DKIM_FILTER_H_
 
 #ifndef lint
-static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.134 2007/11/30 19:42:59 msk Exp $";
+static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.149 2008/01/25 18:45:56 msk Exp $";
 #endif /* !lint */
 
 #define	DKIMF_PRODUCT	"Sendmail DKIM Filter"
-#define	DKIMF_VERSION	"2.4.0"
+#define	DKIMF_VERSION	"2.4.4"
 
 /* libsm */
 #include <sm/gen.h>
@@ -37,6 +37,7 @@ static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.134 2007/11/30 19:4
 #define	CACHESTATSINT	300
 #define	CBINTERVAL	3
 #define CMDLINEOPTS	"a:Ab:c:C:d:DfF:hi:I:k:KlL:m:M:o:p:P:qrRs:S:t:T:u:U:vVx:?"
+#define	DEFMAXHDRSZ	65536
 #define	DEFTIMEOUT	5
 #define	HOSTUNKNOWN	"unknown-host"
 #define	JOBIDUNKNOWN	"(unknown-jobid)"
@@ -44,6 +45,7 @@ static char dkim_filter_h_id[] = "@(#)$Id: dkim-filter.h,v 1.134 2007/11/30 19:4
 #define	MAXADDRESS	256
 #define	MAXARGV		65536
 #define	MAXBUFRSZ	65536
+#define	MAXHDRCNT	64
 #define	MAXHDRLEN	78
 #define	MAXSIGNATURE	1024
 #define	TEMPFILE	"/var/tmp/dkXXXXXX"
