@@ -2,11 +2,11 @@
 **  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: util.c,v 1.55 2007/09/28 17:26:04 msk Exp $
+**  $Id: util.c,v 1.62 2007/10/31 18:34:11 msk Exp $
 */
 
 #ifndef lint
-static char util_c_id[] = "@(#)$Id: util.c,v 1.55 2007/09/28 17:26:04 msk Exp $";
+static char util_c_id[] = "@(#)$Id: util.c,v 1.62 2007/10/31 18:34:11 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -28,6 +28,10 @@ static char util_c_id[] = "@(#)$Id: util.c,v 1.55 2007/09/28 17:26:04 msk Exp $"
 #include <netdb.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <limits.h>
+#ifdef _FFR_REPLACE_RULES
+# include <regex.h>
+#endif /* _FFR_REPLACE_RULES */
 
 /* libsm includes */
 #include <sm/gen.h>
@@ -65,6 +69,10 @@ static char *optlist[] =
 	"POPAUTH",
 #endif /* POPAUTH */
 
+#if QUERY_CACHE
+	"QUERY_CACHE",
+#endif /* QUERY_CACHE */
+
 #if VERIFY_DOMAINKEYS
 	"VERIFY_DOMAINKEYS",
 #endif /* VERIFY_DOMAINKEYS */
@@ -77,9 +85,21 @@ static char *optlist[] =
 	"_FFR_CAPTURE_UNKNOWN_ERRORS",
 #endif /* _FFR_CAPTURE_UNKNOWN_ERRORS */
 
-#if _FFR_QUERY_CACHE
-	"_FFR_QUERY_CACHE",
-#endif /* _FFR_QUERY_CACHE */
+#if _FFR_DIFFHEADERS
+	"_FFR_DIFFHEADERS",
+#endif /* _FFR_DIFFHEADERS */
+
+#if _FFR_DNS_UPGRADE
+	"_FFR_DNS_UPGRADE",
+#endif /* _FFR_DNS_UPGRADE */
+
+#if _FFR_PARSE_TIME
+	"_FFR_PARSE_TIME",
+#endif /* _FFR_PARSE_TIME */
+
+#if _FFR_REPLACE_RULES
+	"_FFR_REPLACE_RULES",
+#endif /* _FFR_REPLACE_RULES */
 
 #if _FFR_REQUIRED_HEADERS
 	"_FFR_REQUIRED_HEADERS",
@@ -89,9 +109,9 @@ static char *optlist[] =
 	"_FFR_SELECT_CANONICALIZATION",
 #endif /* _FFR_SELECT_CANONICALIZATION */
 
-#if _FFR_SELECT_SIGN_HEADERS
-	"_FFR_SELECT_SIGN_HEADERS",
-#endif /* _FFR_SELECT_SIGN_HEADERS */
+#if _FFR_SELECTOR_HEADER
+	"_FFR_SELECTOR_HEADER",
+#endif /* _FFR_SELECTOR_HEADER */
 
 #if _FFR_STATS
 	"_FFR_STATS",
@@ -101,9 +121,50 @@ static char *optlist[] =
 	"_FFR_VBR",
 #endif /* _FFR_VBR */
 
+#if _FFR_ZTAGS
+	"_FFR_ZTAGS",
+#endif /* _FFR_ZTAGS */
+
 	NULL
 };
 
+/* struct dkimf_dstring -- a dynamically-sized string */
+struct dkimf_dstring
+{
+	int			ds_alloc;
+	int			ds_max;
+	int			ds_len;
+	char *			ds_buf;
+};
+
+#ifdef _FFR_REPLACE_RULES
+/*
+**  DKIMF_ISBLANK -- return TRUE iff a string contains only whitespace
+**  
+**  Parameters:
+**  	str -- string to check
+**
+**  Return value:
+**  	TRUE if "str" is either zero-length or contains only whitespace
+*/
+
+bool
+dkimf_isblank(char *str)
+{
+	char *p;
+
+	for (p = str; *p != '\0'; p++)
+	{
+		if (isascii(*p) && isspace(*p))
+			continue;
+
+		return FALSE;
+	}
+
+	return TRUE;
+}
+#endif /* _FFR_REPLACE_RULES */
+
 /*
 **  DKIMF_OPTLIST -- print active FFRs
 **
@@ -686,6 +747,89 @@ dkimf_load_list(FILE *in, char **deflist, struct Peer **list)
 	return TRUE;
 }
 
+#ifdef _FFR_REPLACE_RULES
+/*
+**  DKIMF_LOAD_REPLIST -- load a list of replace patterns
+**
+**  Parameters:
+**  	in -- input stream (must already be open)
+**  	list -- list to be updated
+**
+**  Return value:
+**  	TRUE if successful, FALSE otherwise
+**
+**  Side effects:
+**  	Prints an error message when appropriate.
+*/
+
+bool
+dkimf_load_replist(FILE *in, struct replace **list)
+{
+	int line;
+	int status;
+	char *p;
+	struct replace *newrep;
+	char rule[BUFRSZ + 1];
+
+	assert(in != NULL);
+	assert(list != NULL);
+
+	memset(rule, '\0', sizeof rule);
+
+	while (fgets(rule, sizeof(rule) - 1, in) != NULL)
+	{
+		line++;
+
+		for (p = rule; *p != '\0'; p++)
+		{
+			if (*p == '\n' || *p == '#')
+			{
+				*p = '\0';
+				break;
+			}
+		}
+
+		if (dkimf_isblank(rule))
+			continue;
+
+		newrep = (struct replace *) malloc(sizeof(struct replace));
+		if (newrep == NULL)
+		{
+			fprintf(stderr, "%s: malloc(): %s\n", progname,
+			        strerror(errno));
+			return FALSE;
+		}
+
+		p = strrchr(rule, '\t');
+		if (p == NULL)
+			return FALSE;
+
+		*p = '\0';
+
+		status = regcomp(&newrep->repl_re, rule, 0);
+		if (status != 0)
+		{
+			fprintf(stderr, "%s: regcomp() failed\n", progname);
+			return FALSE;
+		}
+
+		newrep->repl_txt = strdup(p + 1);
+		if (newrep->repl_txt == NULL)
+		{
+			fprintf(stderr, "%s: strdup(): %s\n", progname,
+			        strerror(errno));
+			return FALSE;
+		}
+
+		newrep->repl_next = *list;
+
+		*list = newrep;
+	}
+
+	return TRUE;
+}
+#endif /* _FFR_REPLACE_RULES */
+
 /*
 **  DKIMF_INET_NTOA -- thread-safe inet_ntoa()
 **
@@ -860,6 +1004,339 @@ dkimf_hostlist(char *host, char **list)
 	return FALSE;
 }
 
+/*
+**  DKIMF_DSTRING_RESIZE -- resize a dynamic string (dstring)
+**
+**  Parameters:
+**  	dstr -- DKIMF_DSTRING handle
+**  	len -- number of bytes desired
+**
+**  Return value:
+**  	TRUE iff the resize worked (or wasn't needed)
+**
+**  Notes:
+**  	This will actually ensure that there are "len" bytes available.
+**  	The caller must account for the NULL byte when requesting a
+**  	specific size.
+*/
+
+static bool
+dkimf_dstring_resize(struct dkimf_dstring *dstr, int len)
+{
+	int newsz;
+	char *new;
+
+	assert(dstr != NULL);
+	assert(len > 0);
+
+	if (dstr->ds_alloc >= len)
+		return TRUE;
+
+	/* must resize */
+	for (newsz = dstr->ds_alloc * 2;
+	     newsz < len;
+	     newsz *= 2)
+	{
+		/* impose ds_max limit, if specified */
+		if (dstr->ds_max > 0 && newsz > dstr->ds_max)
+		{
+			if (len <= dstr->ds_max)
+			{
+				newsz = len;
+				break;
+			}
+
+			return FALSE;
+		}
+
+		/* check for overflow */
+		if (newsz > INT_MAX / 2)
+		{
+			/* next iteration will overflow "newsz" */
+			return FALSE;
+		}
+	}
+
+	new = malloc(newsz);
+	if (new == NULL)
+		return FALSE;
+
+	memcpy(new, dstr->ds_buf, dstr->ds_alloc);
+
+	free(dstr->ds_buf);
+
+	dstr->ds_alloc = newsz;
+	dstr->ds_buf = new;
+
+	return TRUE;
+}
+
+/*
+**  DKIMF_DSTRING_NEW -- make a new dstring
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	len -- initial number of bytes
+**  	maxlen -- maximum allowed length (0 == unbounded)
+**
+**  Return value:
+**  	A DKIMF_DSTRING handle, or NULL on failure.
+*/
+
+struct dkimf_dstring *
+dkimf_dstring_new(int len, int maxlen)
+{
+	struct dkimf_dstring *new;
+
+	/* fail on invalid parameters */
+	if ((maxlen > 0 && len > maxlen) || len == 0)
+		return NULL;
+
+	if (len < BUFRSZ)
+		len = BUFRSZ;
+
+	new = malloc(sizeof(struct dkimf_dstring));
+	if (new == NULL)
+		return NULL;
+
+	new->ds_buf = malloc(len);
+	if (new->ds_buf == NULL)
+	{
+		free(new);
+		return NULL;
+	}
+
+	memset(new->ds_buf, '\0', len);
+	new->ds_alloc = len;
+	new->ds_len = 0;
+	new->ds_max = maxlen;
+
+	return new;
+}
+
+/*
+**  DKIMF_DSTRING_FREE -- destroy an existing dstring
+**
+**  Parameters:
+**  	dstr -- DKIMF_DSTRING handle to be destroyed
+**
+**  Return value:
+**  	None.
+*/
+
+void
+dkimf_dstring_free(struct dkimf_dstring *dstr)
+{
+	assert(dstr != NULL);
+
+	free(dstr->ds_buf);
+	free(dstr);
+}
+
+/*
+**  DKIMF_DSTRING_COPY -- copy data into a dstring
+**
+**  Parameters:
+**  	dstr -- DKIMF_DSTRING handle to update
+**  	str -- input string
+**
+**  Return value:
+**  	TRUE iff the copy succeeded.
+**
+**  Side effects:
+**  	The dstring may be resized.
+*/
+
+bool
+dkimf_dstring_copy(struct dkimf_dstring *dstr, char *str)
+{
+	int len;
+
+	assert(dstr != NULL);
+	assert(str != NULL);
+
+	len = strlen(str);
+
+	/* too big? */
+	if (dstr->ds_max > 0 && len >= dstr->ds_max)
+		return FALSE;
+
+	/* fits now? */
+	if (dstr->ds_alloc <= len)
+	{
+		/* nope; try to resize */
+		if (!dkimf_dstring_resize(dstr, len + 1))
+			return FALSE;
+	}
+
+	/* copy */
+	dstr->ds_len = sm_strlcpy(dstr->ds_buf, str, dstr->ds_alloc);
+
+	return TRUE;
+}
+
+/*
+**  DKIMF_DSTRING_CAT -- append data onto a dstring
+**
+**  Parameters:
+**  	dstr -- DKIMF_DSTRING handle to update
+**  	str -- input string
+**
+**  Return value:
+**  	TRUE iff the update succeeded.
+**
+**  Side effects:
+**  	The dstring may be resized.
+*/
+
+bool
+dkimf_dstring_cat(struct dkimf_dstring *dstr, char *str)
+{
+	int len;
+
+	assert(dstr != NULL);
+	assert(str != NULL);
+
+	len = strlen(str) + dstr->ds_len;
+
+	/* too big? */
+	if (dstr->ds_max > 0 && len >= dstr->ds_max)
+		return FALSE;
+
+	/* fits now? */
+	if (dstr->ds_alloc <= len)
+	{
+		/* nope; try to resize */
+		if (!dkimf_dstring_resize(dstr, len + 1))
+			return FALSE;
+	}
+
+	/* append */
+	dstr->ds_len = sm_strlcat(dstr->ds_buf, str, dstr->ds_alloc);
+
+	return TRUE;
+}
+
+/*
+**  DKIMF_DSTRING_CAT1 -- append one byte onto a dstring
+**
+**  Parameters:
+**  	dstr -- DKIMF_DSTRING handle to update
+**  	c -- input character
+**
+**  Return value:
+**  	TRUE iff the update succeeded.
+**
+**  Side effects:
+**  	The dstring may be resized.
+*/
+
+bool
+dkimf_dstring_cat1(struct dkimf_dstring *dstr, int c)
+{
+	int len;
+
+	assert(dstr != NULL);
+
+	len = dstr->ds_len + 1;
+
+	/* too big? */
+	if (dstr->ds_max > 0 && len >= dstr->ds_max)
+		return FALSE;
+
+	/* fits now? */
+	if (dstr->ds_alloc <= len)
+	{
+		/* nope; try to resize */
+		if (!dkimf_dstring_resize(dstr, len + 1))
+			return FALSE;
+	}
+
+	/* append */
+	dstr->ds_buf[dstr->ds_len++] = c;
+	dstr->ds_buf[dstr->ds_len] = '\0';
+
+	return TRUE;
+}
+
+/*
+**  DKIMF_DSTRING_GET -- retrieve data in a dstring
+**
+**  Parameters:
+**  	dstr -- DKIMF_DSTRING handle whose string should be retrieved
+**
+**  Return value:
+**  	Pointer to the NULL-terminated contents of "dstr".
+*/
+
+char *
+dkimf_dstring_get(struct dkimf_dstring *dstr)
+{
+	assert(dstr != NULL);
+
+	return dstr->ds_buf;
+}
+
+/*
+**  DKIMF_DSTRING_LEN -- retrieve length of data in a dstring
+**
+**  Parameters:
+**  	dstr -- DKIMF_DSTRING handle whose string should be retrieved
+**
+**  Return value:
+**  	Number of bytes in a dstring.
+*/
+
+int
+dkimf_dstring_len(struct dkimf_dstring *dstr)
+{
+	assert(dstr != NULL);
+
+	return dstr->ds_len;
+}
+
+/*
+**  DKIMF_DSTRING_BLANK -- clear out the contents of a dstring
+**
+**  Parameters:
+**  	dstr -- DKIMF_DSTRING handle whose string should be cleared
+**
+**  Return value:
+**  	None.
+*/
+
+void
+dkimf_dstring_blank(struct dkimf_dstring *dstr)
+{
+	assert(dstr != NULL);
+
+	dstr->ds_len = 0;
+	dstr->ds_buf[0] = '\0';
+}
+
+/*
+**  DKIMF_DSTRING_CHOP -- truncate contents of a dstring
+**
+**  Parameters:
+**  	dstr -- DKIMF_DSTRING handle whose string should be cleared
+**  	len -- length after which to clobber
+**
+**  Return value:
+**  	None.
+*/
+
+void
+dkimf_dstring_chop(struct dkimf_dstring *dstr, int len)
+{
+	assert(dstr != NULL);
+
+	if (len < dstr->ds_len)
+	{
+		dstr->ds_len = len;
+		dstr->ds_buf[len] = '\0';
+	}
+}
+
 /*
 **  DKIMF_SOCKET_CLEANUP -- try to clean up the socket
 **
