@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char ar_c_id[] = "@(#)$Id: ar.c,v 1.71 2007/07/03 13:54:48 msk Exp $";
+static char ar_c_id[] = "@(#)$Id: ar.c,v 1.72 2007/12/17 23:17:53 msk Exp $";
 #endif /* !lint */
 
 /* OS stuff */
@@ -1216,6 +1216,7 @@ static int
 ar_res_init(AR_LIB new)
 {
 	size_t bytes;
+	SOCKADDR *sa;
 
 	assert(new != NULL);
 
@@ -1268,6 +1269,21 @@ ar_res_init(AR_LIB new)
 #else /* defined(AR_RES_MANUAL) || defined(AF_INET6) */
 	memcpy(new->ar_nsaddrs, ar_res.nsaddr_list,
 	       sizeof(SOCKADDR) * ar_res.nscount);
+
+	/* an address of 0 (INADDR_ANY) should become INADDR_LOOPBACK */
+	for (c = 0; c < new->ar_nscount; c++)
+	{
+		sa = (SOCKADDR *) new->ar_nsaddrs[c];
+		if (sa->sa_family == AF_INET)
+		{
+			struct sockaddr_in *sin;
+
+			sin = (struct sockaddr_in *) sa;
+
+			if (sin->sin_addr.s_addr == INADDR_ANY)
+				sin->sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+		}
+	}
 #endif /* defined(AR_RES_MANUAL) || defined(AF_INET6) */
 
 	return 0;
