@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char manual_c_id[] = "@(#)$Id: manual.c,v 1.4 2007/04/21 18:30:07 msk Exp $";
+static char manual_c_id[] = "@(#)$Id: manual.c,v 1.6 2008/01/05 01:50:18 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -84,7 +84,29 @@ ar_res_parse(int *nscount, struct sockaddr_in *out, int *retry, int *retrans)
 
 	f = fopen(_PATH_RESCONF, "r");
 	if (f == NULL)
-		return -1;
+	{
+		struct sockaddr *sa;
+		struct sockaddr_in *sin;
+
+		/* apply defaults */
+#ifdef AF_INET6
+		sa = (struct sockaddr *) &out[0];
+		sin = (struct sockaddr_in *) sa;
+		sin->sin_family = AF_INET;
+		sin->sin_port = srv->s_port;
+		sin->sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+#else /* AF_INET6 */
+		out[0].sin_family = AF_INET;
+		out[0].sin_port = srv->s_port;
+		out[0].sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+#endif /* AF_INET6 */
+
+		ns = 1;
+
+		*nscount = ns;
+
+		return 0;
+	}
 
 	clearerr(f);
 	while (fgets(buf, sizeof buf, f) != NULL)
@@ -181,6 +203,28 @@ ar_res_parse(int *nscount, struct sockaddr_in *out, int *retry, int *retrans)
 
 	*retry = RES_RETRY;
 	*retrans = RES_TIMEOUT;
+
+	/* if no "nameserver" lines were found, add a default one */
+	if (ns == 0)
+	{
+#ifdef AF_INET6
+		struct sockaddr *sa;
+		struct sockaddr_in *sin;
+
+		sa = (struct sockaddr *) &out[0];
+		sin = (struct sockaddr_in *) sa;
+		sin->sin_family = AF_INET;
+		sin->sin_port = srv->s_port;
+		sin->sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+#else /* AF_INET6 */
+		out[0].sin_family = AF_INET;
+		out[0].sin_port = srv->s_port;
+		out[0].sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+#endif /* AF_INET6 */
+
+		ns = 1;
+	}
+
 	*nscount = ns;
 
 	return 0;
