@@ -1,10 +1,10 @@
 /*
-**  Copyright (c) 2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2007, 2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
 #ifndef lint
-static char dkim_canon_c_id[] = "@(#)$Id: dkim-canon.c,v 1.28 2007/11/15 22:43:09 msk Exp $";
+static char dkim_canon_c_id[] = "@(#)$Id: dkim-canon.c,v 1.37 2008/01/02 21:20:50 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -30,6 +30,9 @@ static char dkim_canon_c_id[] = "@(#)$Id: dkim-canon.c,v 1.28 2007/11/15 22:43:0
 #define	CRLF	"\r\n"
 #define	SP	" "
 
+/* macros */
+#define	DKIM_ISLWSP(x)	((x) == 011 || (x) == 013 || (x) == 014 || (x) == 040)
+
 /* prototypes */
 extern void dkim_error __P((DKIM *, const char *, ...));
 
@@ -168,18 +171,21 @@ dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
                   bool crlf)
 {
 	bool space;
-	bool firstword;
-	bool colon;
 	int n;
 	u_char *p;
+	u_char *tmp;
+	u_char *end;
+	u_char tmpbuf[BUFRSZ];
 
 	assert(canon != NULL);
 	assert(hdr != NULL);
 
+	tmp = tmpbuf;
+	end = tmpbuf + sizeof tmpbuf - 1;
+
 	if (dkim->dkim_canonbuf == NULL)
 	{
-		dkim->dkim_canonbuf = dkim_dstring_new(dkim,
-		                                       strlen(hdr->hdr_text),
+		dkim->dkim_canonbuf = dkim_dstring_new(dkim, hdr->hdr_textlen,
 		                                       0);
 		if (dkim->dkim_canonbuf == NULL)
 			return DKIM_STAT_NORESOURCE;
@@ -196,15 +202,14 @@ dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
 	switch (canon->canon_canon)
 	{
 	  case DKIM_CANON_SIMPLE:
-		if (!dkim_dstring_cat(dkim->dkim_canonbuf, hdr->hdr_text) ||
-		    (crlf && !dkim_dstring_cat(dkim->dkim_canonbuf, CRLF)))
+		if (!dkim_dstring_catn(dkim->dkim_canonbuf,
+		                       hdr->hdr_text, hdr->hdr_textlen) ||
+		    (crlf && !dkim_dstring_catn(dkim->dkim_canonbuf, CRLF, 2)))
 			return DKIM_STAT_NORESOURCE;
 		break;
 
 	  case DKIM_CANON_RELAXED:
-		space = FALSE;				/* just saw a space */
-		firstword = FALSE;			/* in/past first word */
-		colon = FALSE;				/* past colon */
+		/* process header field name (before colon) first */
 		for (p = hdr->hdr_text; *p != '\0'; p++)
 		{
 			/*
@@ -212,26 +217,55 @@ dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
 			**  of the first word.
 			*/
 
-			if (isascii(*p) && isspace(*p) &&
-			    (!colon || !firstword))
-				continue;
-
-			/*
-			**  Discard spaces other than the first one after
-			**  the first word.
-			*/
-
-			if (isascii(*p) && isspace(*p) && space && firstword)
-				continue;
-
-			/*
-			**  The first space found after a colon that gets
-			**  this far marks the end of a word.
-			*/
-
-			if (isascii(*p) && isspace(*p) && colon && !space)
+			if (isascii(*p))
 			{
+				/* discard spaces */
+				if (isspace(*p))
+					continue;
+
+				/* convert to lowercase */
+				if (isupper(*p))
+					*tmp++ = tolower(*p);
+				else
+					*tmp++ = *p;
+			}
+			else
+			{
+				*tmp++ = *p;
+			}
+
+			/* reaching the end of the cache buffer, flush it */
+			if (tmp == end)
+			{
+				*tmp = '\0';
+
+				if (!dkim_dstring_catn(dkim->dkim_canonbuf,
+				                       tmpbuf, tmp - tmpbuf))
+					return DKIM_STAT_NORESOURCE;
+
+				tmp = tmpbuf;
+			}
+			
+			if (*p == ':')
+			{
+				p++;
+				break;
+			}
+		}
+
+		/* skip all spaces before first word */
+		while (*p != '\0' && isascii(*p) && isspace(*p))
+			p++;
+
+		space = FALSE;				/* just saw a space */
+
+		for ( ; *p != '\0'; p++)
+		{
+			if (isascii(*p) && isspace(*p))
+			{
+				/* mark that there was a space and continue */
 				space = TRUE;
+
 				continue;
 			}
 
@@ -240,35 +274,53 @@ dkim_canon_header(DKIM *dkim, DKIM_CANON *canon, struct dkim_header *hdr,
 			**  If there's a stored space, use it up.
 			*/
 
-			if ((!isascii(*p) || !isspace(*p)) &&
-			    firstword && space)
+			if (space)
 			{
-				if (!dkim_dstring_cat1(dkim->dkim_canonbuf,
-				                       ' '))
-					return DKIM_STAT_NORESOURCE;
+				*tmp++ = ' ';
+
+				/* flush buffer? */
+				if (tmp == end)
+				{
+					*tmp = '\0';
+
+					if (!dkim_dstring_catn(dkim->dkim_canonbuf,
+					                       tmpbuf,
+					                       tmp - tmpbuf))
+						return DKIM_STAT_NORESOURCE;
+
+					tmp = tmpbuf;
+				}
+
 				space = FALSE;
 			}
 
 			/* copy the byte */
-			if (!dkim_dstring_cat1(dkim->dkim_canonbuf, *p))
-				return DKIM_STAT_NORESOURCE;
+			*tmp++ = *p;
 
-			/*
-			**  If this was the first colon, remember that we
-			**  saw it.  Otherwise, anything that got this
-			**  far after the first colon means we're in or
-			**  past the first word.
-			*/
+			/* flush buffer? */
+			if (tmp == end)
+			{
+				*tmp = '\0';
 
-			if (*p == ':' && !colon)
-				colon = TRUE;
-			else if (colon)
-				firstword = TRUE;
+				if (!dkim_dstring_catn(dkim->dkim_canonbuf,
+				                       tmpbuf, tmp - tmpbuf))
+					return DKIM_STAT_NORESOURCE;
+
+				tmp = tmpbuf;
+			}
 		}
 
-		dkim_lowerhdr(dkim_dstring_get(dkim->dkim_canonbuf));
+		/* flush any cached data */
+		if (tmp != tmpbuf)
+		{
+			*tmp = '\0';
 
-		if (crlf && !dkim_dstring_cat(dkim->dkim_canonbuf, CRLF))
+			if (!dkim_dstring_catn(dkim->dkim_canonbuf,
+			                       tmpbuf, tmp - tmpbuf))
+				return DKIM_STAT_NORESOURCE;
+		}
+
+		if (crlf && !dkim_dstring_catn(dkim->dkim_canonbuf, CRLF, 2))
 			return DKIM_STAT_NORESOURCE;
 
 		break;
@@ -329,16 +381,15 @@ dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
                       int nptrs)
 {
 	u_char *bar;
-	u_char *colon;
 	char *ctx;
+	char *colon;
 	int c;
 	int n;
 	int m;
 	size_t len;
-	size_t hlen;
 	struct dkim_header *hdr;
-	struct dkim_header *lhdrs[MAXHDRCNT];
-	u_char *hdrs[MAXHDRCNT];
+	struct dkim_header **lhdrs;
+	u_char **hdrs;
 
 	assert(dkim != NULL);
 	assert(ptrs != NULL);
@@ -369,15 +420,33 @@ dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
 		{
 			dkim_error(dkim, "unable to allocate %d bytes(s)",
 			           DKIM_MAXHEADER);
+
 			return -1;
 		}
 	}
+
 	sm_strlcpy(dkim->dkim_hdrlist, hdrlist, DKIM_MAXHEADER);
 
 	/* mark all headers as not used */
 	for (hdr = dkim->dkim_hhead; hdr != NULL; hdr = hdr->hdr_next)
 		hdr->hdr_flags &= ~DKIM_HDR_SIGNED;
 
+	n = dkim->dkim_hdrcnt * sizeof(struct dkim_header *);
+	lhdrs = DKIM_MALLOC(dkim, n);
+	if (lhdrs == NULL)
+		return -1;
+
+	n = 1;
+	for (colon = dkim->dkim_hdrlist; *colon != '\0'; colon++)
+	{
+		if (*colon == ':')
+			n++;
+	}
+	n = dkim->dkim_hdrcnt * n;
+	hdrs = DKIM_MALLOC(dkim, n);
+	if (hdrs == NULL)
+		return -1;
+
 	n = 0;
 
 	/* make a split-out copy of hdrlist */
@@ -386,21 +455,18 @@ dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
 	     bar != NULL;
 	     bar = strtok_r(NULL, ":", &ctx))
 	{
-		if (n >= MAXHDRCNT)
-		{
-			dkim_error(dkim, "too many headers (max %d)",
-			           MAXHDRCNT);
-			return -1;
-		}
-
-		hdrs[n] = bar;
+		hdrs[n] = (u_char *) bar;
 		n++;
 	}
 
 	/* bounds check */
-	if (n >= nptrs)
+	if (n > nptrs)
 	{
-		dkim_error(dkim, "too many headers (max %d)", MAXHDRCNT);
+		dkim_error(dkim, "too many headers (max %d)", nptrs);
+
+		DKIM_FREE(dkim, lhdrs);
+		DKIM_FREE(dkim, hdrs);
+
 		return -1;
 	}
 
@@ -420,27 +486,9 @@ dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
 			if (hdr->hdr_flags & DKIM_HDR_SIGNED)
 				continue;
 
-			colon = strchr(hdr->hdr_text, ':');
-			if (colon != NULL)
-			{
-				hlen = colon - hdr->hdr_text;
-
-				/* ignore spaces before the colon */
-				while (hlen > 0 &&
-				       isascii(hdr->hdr_text[hlen - 1]) &&
-				       isspace(hdr->hdr_text[hlen - 1]))
-					hlen--;
-
-				if (len == hlen &&
-				    strncasecmp(hdr->hdr_text, hdrs[c],
-				                hlen) == 0)
-					lhdrs[c] = hdr;
-			}
-			else
-			{
-				if (strcasecmp(hdr->hdr_text, hdrs[c]) == 0)
-					lhdrs[c] = hdr;
-			}
+			if (len == hdr->hdr_namelen &&
+			    strncasecmp(hdr->hdr_text, hdrs[c], len) == 0)
+				lhdrs[c] = hdr;
 		}
 
 		if (lhdrs[c] != NULL)
@@ -458,6 +506,9 @@ dkim_canon_selecthdrs(DKIM *dkim, u_char *hdrlist, struct dkim_header **ptrs,
 		}
 	}
 
+	DKIM_FREE(dkim, lhdrs);
+	DKIM_FREE(dkim, hdrs);
+
 	return m;
 }
 
@@ -732,25 +783,40 @@ dkim_add_canon(DKIM *dkim, bool hdr, dkim_canon_t canon, int hashtype,
 DKIM_STAT
 dkim_canon_runheaders(DKIM *dkim, bool signing)
 {
+	u_char savechar;
 	int c;
 	int n;
 	int in;
 	int nhdrs;
 	int last = '\0';
 	DKIM_STAT status;
+	u_char *tmp;
+	u_char *end;
 	DKIM_CANON *cur;
 	u_char *p;
 	struct dkim_header *hdr;
-	struct dkim_header *hdrset[MAXHDRCNT];
+	struct dkim_header **hdrset;
 	struct dkim_header tmphdr;
+	u_char tmpbuf[BUFRSZ];
 
 	assert(dkim != NULL);
 
+	tmp = tmpbuf;
+	end = tmpbuf + sizeof tmpbuf - 1;
+
+	n = dkim->dkim_hdrcnt * sizeof(struct dkim_header *);
+	hdrset = DKIM_MALLOC(dkim, n);
+	if (hdrset == NULL)
+		return DKIM_STAT_NORESOURCE;
+
 	if (dkim->dkim_hdrbuf == NULL)
 	{
 		dkim->dkim_hdrbuf = dkim_dstring_new(dkim, BUFRSZ, MAXBUFRSZ);
 		if (dkim->dkim_hdrbuf == NULL)
+		{
+			DKIM_FREE(dkim, hdrset);
 			return DKIM_STAT_NORESOURCE;
+		}
 	}
 	else
 	{
@@ -786,18 +852,19 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 				nhdrs = dkim_canon_selecthdrs(dkim,
 				                              cur->canon_hdrlist,
 				                              hdrset,
-				                              MAXHDRCNT);
+				                              dkim->dkim_hdrcnt);
+
 				if (nhdrs == -1)
 				{
 					dkim_error(dkim,
 					           "dkim_canon_selecthdrs() failed during canonicalization");
+					DKIM_FREE(dkim, hdrset);
 					return DKIM_STAT_INTERNAL;
 				}
 			}
 		}
 		else
 		{
-			u_char *colon;
 			DKIM_LIB *lib;
 			char *tmp;
 
@@ -811,50 +878,51 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 			     hdr != NULL && nhdrs < MAXHEADERS;
 			     hdr = hdr->hdr_next)
 			{
-				colon = strchr(hdr->hdr_text, ':');
-				if (colon != NULL)
-					*colon = '\0';
-
 				if (!lib->dkiml_signre)
 				{
 					tmp = dkim_dstring_get(dkim->dkim_hdrbuf);
 
 					if (tmp[0] != '\0')
 					{
-						dkim_dstring_cat(dkim->dkim_hdrbuf,
-						                 ":");
+						dkim_dstring_cat1(dkim->dkim_hdrbuf,
+						                 ':');
 					}
 
-					dkim_dstring_cat(dkim->dkim_hdrbuf,
-					                 hdr->hdr_text);
-					if (colon != NULL)
-						*colon = ':';
+					dkim_dstring_catn(dkim->dkim_hdrbuf,
+					                  hdr->hdr_text,
+					                  hdr->hdr_namelen);
 					continue;
 				}
 
+				/* could be space, could be colon ... */
+				savechar = hdr->hdr_text[hdr->hdr_namelen];
+
+				/* terminate the header field name and test */
+				hdr->hdr_text[hdr->hdr_namelen] = '\0';
 				status = regexec(&lib->dkiml_hdrre,
 				                 hdr->hdr_text, 0, NULL, 0);
 
+				/* restore the character */
+				hdr->hdr_text[hdr->hdr_namelen] = savechar;
+
 				if (status == 0)
 				{
 					tmp = dkim_dstring_get(dkim->dkim_hdrbuf);
 
 					if (tmp[0] != '\0')
 					{
-						dkim_dstring_cat(dkim->dkim_hdrbuf,
-						                 ":");
+						dkim_dstring_cat1(dkim->dkim_hdrbuf,
+						                 ':');
 					}
 
-					dkim_dstring_cat(dkim->dkim_hdrbuf,
-					                 hdr->hdr_text);
+					dkim_dstring_catn(dkim->dkim_hdrbuf,
+					                  hdr->hdr_text,
+							  hdr->hdr_namelen);
 				}
 				else
 				{
 					assert(status == REG_NOMATCH);
 				}
-
-				if (colon != NULL)
-					*colon = ':';
 			}
 
 			memset(hdrset, '\0', sizeof hdrset);
@@ -862,11 +930,14 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 			/* do header selection */
 			nhdrs = dkim_canon_selecthdrs(dkim,
 			                              dkim_dstring_get(dkim->dkim_hdrbuf),
-			                              hdrset, MAXHDRCNT);
+			                              hdrset,
+			                              dkim->dkim_hdrcnt);
+
 			if (nhdrs == -1)
 			{
 				dkim_error(dkim,
 				           "dkim_canon_selecthdrs() failed during canonicalization");
+				DKIM_FREE(dkim, hdrset);
 				return DKIM_STAT_INTERNAL;
 			}
 		}
@@ -880,7 +951,10 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 				status = dkim_canon_header(dkim, cur,
 				                           hdrset[c], TRUE);
 				if (status != DKIM_STAT_OK)
+				{
+					DKIM_FREE(dkim, hdrset);
 					return status;
+				}
 			}
 		}
 
@@ -897,6 +971,8 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 
 		dkim_dstring_blank(dkim->dkim_hdrbuf);
 
+		tmp = tmpbuf;
+
 		n = 0;
 		in = '\0';
 		for (p = cur->canon_sigheader->hdr_text; *p != '\0'; p++)
@@ -913,12 +989,38 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 			if (in == '\0' && *p == '=')
 				in = last;
 
-			dkim_dstring_cat1(dkim->dkim_hdrbuf, *p);
+			*tmp++ = *p;
+
+			/* flush buffer? */
+			if (tmp == end)
+			{
+				*tmp = '\0';
+
+				if (!dkim_dstring_catn(dkim->dkim_hdrbuf,
+				                       tmpbuf, tmp - tmpbuf))
+					return DKIM_STAT_NORESOURCE;
+
+				tmp = tmpbuf;
+			}
+
 			last = *p;
 		}
 
+		/* flush anything cached */
+		if (tmp != tmpbuf)
+		{
+			*tmp = '\0';
+
+			if (!dkim_dstring_catn(dkim->dkim_hdrbuf,
+			                       tmpbuf, tmp - tmpbuf))
+				return DKIM_STAT_NORESOURCE;
+		}
+
 		/* canonicalize */
 		tmphdr.hdr_text = dkim_dstring_get(dkim->dkim_hdrbuf);
+		tmphdr.hdr_namelen = cur->canon_sigheader->hdr_namelen;
+		tmphdr.hdr_colon = tmphdr.hdr_text + (cur->canon_sigheader->hdr_colon - cur->canon_sigheader->hdr_text);
+		tmphdr.hdr_textlen = dkim_dstring_len(dkim->dkim_hdrbuf);
 		tmphdr.hdr_flags = 0;
 		tmphdr.hdr_next = NULL;
 
@@ -966,6 +1068,8 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 		cur->canon_done = TRUE;
 	}
 
+	DKIM_FREE(dkim, hdrset);
+
 	return DKIM_STAT_OK;
 }
 
@@ -1010,6 +1114,9 @@ dkim_canon_signature(DKIM *dkim, struct dkim_header *hdr)
 		/* prepare the data */
 		dkim_dstring_copy(dkim->dkim_hdrbuf, hdr->hdr_text);
 		tmphdr.hdr_text = dkim_dstring_get(dkim->dkim_hdrbuf);
+		tmphdr.hdr_namelen = hdr->hdr_namelen;
+		tmphdr.hdr_colon = tmphdr.hdr_text + (hdr->hdr_colon - hdr->hdr_text);
+		tmphdr.hdr_textlen = dkim_dstring_len(dkim->dkim_hdrbuf);
 		tmphdr.hdr_flags = 0;
 		tmphdr.hdr_next = NULL;
 		if (cur->canon_canon == DKIM_CANON_RELAXED)
@@ -1220,8 +1327,8 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 				else
 				{
 					/* non-space after a space */
-					if (dkim_islwsp(cur->canon_lastchar) &&
-					    !dkim_islwsp(*p) && *p != '\r')
+					if (DKIM_ISLWSP(cur->canon_lastchar) &&
+					    !DKIM_ISLWSP(*p) && *p != '\r')
 					{
 						if (cur->canon_blanks > 0)
 							dkim_canon_flushblanks(cur);
@@ -1232,8 +1339,8 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 					}
 
 					/* space after a non-space */
-					else if (!dkim_islwsp(cur->canon_lastchar) &&
-					         dkim_islwsp(*p))
+					else if (!DKIM_ISLWSP(cur->canon_lastchar) &&
+					         DKIM_ISLWSP(*p))
 					{
 						if (cur->canon_blanks > 0)
 							dkim_canon_flushblanks(cur);
@@ -1244,8 +1351,8 @@ dkim_canon_bodychunk(DKIM *dkim, u_char *buf, size_t buflen)
 					}
 
 					/* space after a space */
-					else if (dkim_islwsp(cur->canon_lastchar) &&
-					         dkim_islwsp(*p))
+					else if (DKIM_ISLWSP(cur->canon_lastchar) &&
+					         DKIM_ISLWSP(*p))
 					{
 						/* XXX -- fix logic */
 						;
