@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.19 2007/09/07 00:28:57 msk Exp $";
+static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.23 2007/11/02 20:15:24 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -33,9 +33,9 @@ static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.19 2007/09/07 00:28
 #include "dkim.h"
 #include "dkim-types.h"
 #include "dkim-policy.h"
-#ifdef _FFR_QUERY_CACHE
+#ifdef QUERY_CACHE
 # include "dkim-cache.h"
-#endif /* _FFR_QUERY_CACHE */
+#endif /* QUERY_CACHE */
 
 /* prototypes */
 extern void dkim_error __P((DKIM *, const char *, ...));
@@ -59,7 +59,8 @@ extern void dkim_error __P((DKIM *, const char *, ...));
 **  	qstatus -- query result code (DNS-style)
 **
 **  Return value:
-**  	0 -- query completed
+**  	1 -- query completed, answer returned in "buf"
+**  	0 -- query completed, no answer available
 **  	-1 -- failure
 */
 
@@ -123,7 +124,7 @@ dkim_get_policy_file(DKIM *dkim, unsigned char *query, unsigned char *buf,
 
 			fclose(f);
 
-			return 0;
+			return 1;
 		}
 	}
 
@@ -147,6 +148,7 @@ dkim_get_policy_file(DKIM *dkim, unsigned char *query, unsigned char *buf,
 **  Parameters:
 **  	dkim -- DKIM handle
 **  	query -- query to execute
+**  	usemx -- use an MX record?
 **  	buf -- buffer into which to write policy
 **  	buflen -- number of bytes available at "buf"
 **  	qstatus -- query result code (DNS-style)
@@ -158,19 +160,20 @@ dkim_get_policy_file(DKIM *dkim, unsigned char *query, unsigned char *buf,
 */
 
 int
-dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
-                    size_t buflen, int *qstatus)
+dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool usemx,
+                    unsigned char *buf, size_t buflen, int *qstatus)
 {
 	int qdcount;
 	int ancount;
 	int status;
 	int n;
 	int c;
+	int qtype;
 	int type = -1;
 	int class = -1;
-#ifdef _FFR_QUERY_CACHE
+#ifdef QUERY_CACHE
 	int ttl;
-#endif /* _FFR_QUERY_CACHE */
+#endif /* QUERY_CACHE */
 	size_t anslen;
 #if USE_ARLIB
 	AR_LIB ar;
@@ -196,7 +199,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 
 	lib = dkim->dkim_libhandle;
 
-#ifdef _FFR_QUERY_CACHE
+#ifdef QUERY_CACHE
 	if (lib->dkiml_cache != NULL)
 	{
 		int err = 0;
@@ -214,7 +217,9 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 		}
 		/* XXX -- do something with errors here */
 	}
-#endif /* _FFR_QUERY_CACHE */
+#endif /* QUERY_CACHE */
+
+	qtype = (usemx ? T_MX : T_TXT);
 
 #if USE_ARLIB
 # ifdef _FFR_DNS_UPGRADE
@@ -234,14 +239,14 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 		timeout.tv_sec = dkim->dkim_timeout;
 		timeout.tv_usec = 0;
 
-		q = ar_addquery(ar, query, C_IN, T_TXT, MAXCNAMEDEPTH, ansbuf,
+		q = ar_addquery(ar, query, C_IN, qtype, MAXCNAMEDEPTH, ansbuf,
 		                sizeof ansbuf, &arerror,
 		                dkim->dkim_timeout == 0 ? NULL : &timeout);
 		if (q == NULL)
 		{
 			dkim_error(dkim, "ar_addquery() for `%s' failed",
 			           query);
-			return DKIM_STAT_INTERNAL;
+			return -1;
 		}
 
 		if (lib->dkiml_dns_callback == NULL)
@@ -282,7 +287,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 	timeout.tv_sec = dkim->dkim_timeout;
 	timeout.tv_usec = 0;
 
-	q = ar_addquery(ar, query, C_IN, T_TXT, MAXCNAMEDEPTH, ansbuf,
+	q = ar_addquery(ar, query, C_IN, qtype, MAXCNAMEDEPTH, ansbuf,
 	                sizeof ansbuf, &arerror,
 	                dkim->dkim_timeout == 0 ? NULL : &timeout);
 	if (q == NULL)
@@ -315,7 +320,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 # endif /* _FFR_DNS_UPGRADE */
 #else /* USE_ARLIB */
 
-	status = res_query(query, C_IN, T_TXT, ansbuf, sizeof ansbuf);
+	status = res_query(query, C_IN, qtype, ansbuf, sizeof ansbuf);
 
 #endif /* USE_ARLIB */
 
@@ -401,7 +406,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 		GETSHORT(class, cp);
 	}
 
-	if (type != T_TXT || class != C_IN)
+	if (type != qtype || class != C_IN)
 	{
 		dkim_error(dkim, "`%s' unexpected reply class/type", query);
 		return -1;
@@ -424,7 +429,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 	if (ancount == 0)
 		return 0;
 
-	/* walk through the answers looking for a TXT record */
+	/* walk through the answers looking for the right record */
 	while (--ancount >= 0 && cp < eom)
 	{
 		/* grab the label, even though we know what we asked... */
@@ -456,7 +461,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 			cp += n;
 			continue;
 		}
-		else if (type != T_TXT)
+		else if (type != qtype)
 		{
 			/* reject anything not valid (stupid wildcards) */
 			dkim_error(dkim, "`%s' unexpected reply class/type",
@@ -464,22 +469,31 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 			return -1;
 		}
 
-		/* process a TXT */
+		/* process it */
 		break;
 	}
 
 	if (ancount < 0)
 	{
 		dkim_error(dkim, "`%s' reply was unresolved CNAME", query);
-		return DKIM_STAT_INTERNAL;
+		return -1;
 	}
 
-#ifdef _FFR_QUERY_CACHE
+	/*
+	**  If we're querying something other than a T_TXT here, it was
+	**  used by part of the policy algorithm merely to validate the
+	**  existence of a domain; the payload isn't really required.
+	*/
+
+	if (usemx)
+		return 1;
+
+#ifdef QUERY_CACHE
 	GETLONG(ttl, cp);
-#else /* _FFR_QUERY_CACHE */
+#else /* QUERY_CACHE */
 	/* skip the TTL */
 	cp += INT32SZ;
-#endif /* _FFR_QUERY_CACHE */
+#endif /* QUERY_CACHE */
 
 	/* get payload length */
 	if (cp + INT16SZ > eom)
@@ -511,7 +525,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 		}
 	}
 
-#ifdef _FFR_QUERY_CACHE
+#ifdef QUERY_CACHE
 	if (dkim->dkim_libhandle->dkiml_cache != NULL)
 	{
 		int err = 0;
@@ -520,9 +534,9 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, unsigned char *buf,
 		                           query, outbuf, ttl, &err);
 		/* XXX -- do something with errors here */
 	}
-#endif /* _FFR_QUERY_CACHE */
+#endif /* QUERY_CACHE */
 
 	sm_strlcpy(buf, outbuf, buflen);
 
-	return 0;
+	return 1;
 }
