@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.29 2007/10/02 22:36:58 msk Exp $";
+static char dkim_tables_c_id[] = "@(#)$Id: dkim-tables.c,v 1.32 2007/11/28 18:49:49 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -35,13 +35,21 @@ static struct chartable prv_keyparams[] =	/* key parameters */
 };
 struct chartable *keyparams = prv_keyparams;
 
-static struct chartable prv_policyflags[] =	/* policy flags */
+static struct nametable prv_policyflags[] =	/* policy flags */
 {
-	{ 'y',		DKIM_PFLAG_TEST },
-	{ 's',		DKIM_PFLAG_NOSUBDOMAIN },
-	{ '\0',		-1 }
+	{ "y",		DKIM_PFLAG_TEST },
+	{ "s",		DKIM_PFLAG_NOSUBDOMAIN },
+	{ NULL,		-1 }
 };
-struct chartable *policyflags = prv_policyflags;
+struct nametable *policyflags = prv_policyflags;
+
+static struct nametable prv_keyflags[] =	/* policy flags */
+{
+	{ "y",		DKIM_SIGFLAG_TESTKEY },
+	{ "s",		DKIM_SIGFLAG_NOSUBDOMAIN },
+	{ NULL,		-1 }
+};
+struct nametable *keyflags = prv_keyflags;
 
 static struct nametable prv_policyparams[] =	/* policy parameters */
 {
@@ -62,13 +70,13 @@ static struct nametable prv_policies[] =	/* policies */
 };
 struct nametable *policies = prv_policies;
 
-static struct nametable prv_handling[] =	/* SSP recommended handling */
+static struct nametable prv_handlings[] =	/* SSP recommended handling */
 {
 	{ "process",	DKIM_HANDLING_PROCESS },
 	{ "deny",	DKIM_HANDLING_DENY },
 	{ NULL,		-1 }
 };
-struct nametable *handling = prv_handling;
+struct nametable *handlings = prv_handlings;
 
 static struct nametable prv_policyresults[] =	/* policy results */
 {
@@ -207,7 +215,8 @@ static struct nametable prv_sigerrors[] =	/* signature parsing errors */
 	{ "body hash missing",			DKIM_SIGERROR_MISSING_BH },
 	{ "body hash empty",			DKIM_SIGERROR_EMPTY_BH },
 	{ "body hash corrupt",			DKIM_SIGERROR_CORRUPT_BH },
-	{ "signature verification failed",		DKIM_SIGERROR_BADSIG },
+	{ "signature verification failed",	DKIM_SIGERROR_BADSIG },
+	{ "unauthorized subdomain",		DKIM_SIGERROR_SUBDOMAIN },
 	{ NULL,					-1 },
 };
 struct nametable *sigerrors = prv_sigerrors;
