@@ -7,7 +7,7 @@
 #define _DKIM_TABLES_H_
 
 #ifndef lint
-static char dkim_tables_h_id[] = "@(#)$Id: dkim-tables.h,v 1.12 2007/09/20 22:08:05 msk Exp $";
+static char dkim_tables_h_id[] = "@(#)$Id: dkim-tables.h,v 1.14 2007/11/28 18:49:49 msk Exp $";
 #endif /* !lint */
 
 /* structures */
@@ -25,12 +25,11 @@ struct chartable
 
 /* tables */
 struct chartable *keyparams;
-struct chartable *policyflags;
 
 struct nametable *algorithms;
 struct nametable *canonicalizations;
 struct nametable *hashes;
-struct nametable *handling;
+struct nametable *handlings;
 struct nametable *keytypes;
 struct nametable *policies;
 struct nametable *policyparams;
@@ -40,6 +39,8 @@ struct nametable *results;
 struct nametable *settypes;
 struct nametable *sigerrors;
 struct nametable *sigparams;
+struct nametable *policyflags;
+struct nametable *keyflags;
 
 /* prototypes */
 extern const char *dkim_code_to_name __P((struct nametable *tbl,
