@@ -1,5 +1,5 @@
 /*
-**  Copyright (c) 2005-2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2005-2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
@@ -7,7 +7,7 @@
 #define _DKIM_H_
 
 #ifndef lint
-static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.154 2007/11/27 21:10:51 msk Exp $";
+static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.160 2008/01/24 01:54:28 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -31,7 +31,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.154 2007/11/27 21:10:51 msk Exp $
 **  	pp == patch number
 */
 
-#define	DKIM_LIB_VERSION	0x02040000
+#define	DKIM_LIB_VERSION	0x02040400
 
 /* the basics */
 #ifndef NULL
@@ -66,7 +66,6 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.154 2007/11/27 21:10:51 msk Exp $
 #define MAXADDRESS		256	/* biggest user@host we accept */
 #define	MAXBUFRSZ		65536	/* max temp buffer size */
 #define MAXCNAMEDEPTH		3	/* max. CNAME recursion we allow */
-#define MAXHDRCNT		64	/* max. headers signed */
 #define MAXHEADERS		32768	/* buffer for caching headers */
 #define MAXLABELS		16	/* max. labels we allow */
 #define MAXPOLICYDEPTH		1	/* policy recursion we allow */
@@ -86,9 +85,15 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.154 2007/11/27 21:10:51 msk Exp $
 #define	DKIM_VERSION_SIGOLD	"0.5"	/* old version token for sigs */
 
 /* headers */
+#define	DKIM_DATEHEADER		"Date"	/* Date: header */
 #define	DKIM_FROMHEADER		"From"	/* From: header */
 #define	DKIM_SIGNHEADER		"DKIM-Signature"
 					/* DKIM signature header */
+
+#define	DKIM_DATEHEADER_LEN	(sizeof(DKIM_DATEHEADER) - 1)
+#define	DKIM_FROMHEADER_LEN	(sizeof(DKIM_FROMHEADER) - 1)
+#define	DKIM_SIGNHEADER_LEN	(sizeof(DKIM_SIGNHEADER) - 1)
+
 #define	DKIM_HDRMARGIN		75	/* "standard" header margin */
 
 /* special DNS tokens */
@@ -463,6 +468,13 @@ typedef struct dkim_siginfo DKIM_SIGINFO;
 #define DKIM_SIGBH_MATCH		0
 #define DKIM_SIGBH_MISMATCH		1
 
+/*
+**  DKIM_PSTATE -- policy query state
+*/
+
+struct dkim_pstate;
+typedef struct dkim_pstate DKIM_PSTATE;
+
 #ifdef _FFR_DIFFHEADERS
 /*
 **  DKIM_HDRDIFF -- header differences
@@ -1157,6 +1169,7 @@ extern void dkim_sig_ignore __P((DKIM_SIGINFO *siginfo));
 **  	susp -- suspicious message flag (returned)
 **  	pcode -- discovered policy (returned)
 **  	hcode -- recommended suspicious message handling (returned)
+**  	pstate -- state, for re-entrancy (updated; can be NULL)
 **
 **  Return value:
 **  	A DKIM_STAT_* constant.
@@ -1164,7 +1177,8 @@ extern void dkim_sig_ignore __P((DKIM_SIGINFO *siginfo));
 
 extern DKIM_STAT dkim_policy __P((DKIM *dkim, bool *test, bool *susp,
                                   dkim_policy_t *pcode,
-                                  dkim_handling_t *hcode));
+                                  dkim_handling_t *hcode,
+                                  DKIM_PSTATE *pstate));
 
 /*
 **  DKIM_SIG_PROCESS -- process a signature
