@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char t_test55_c_id[] = "@(#)$Id: t-test55.c,v 1.6 2007/07/27 05:39:47 msk Exp $";
+static char t_test55_c_id[] = "@(#)$Id: t-test55.c,v 1.7 2008/01/16 18:47:55 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -22,6 +22,8 @@ static char t_test55_c_id[] = "@(#)$Id: t-test55.c,v 1.6 2007/07/27 05:39:47 msk
 
 #define	MAXHEADER	4096
 
+#define	GIBBERISHLEN	(sizeof(GIBBERISH) - 1)
+
 #define	SIG1 "v=1; a=rsa-sha256; c=relaxed/simple; d=example.com; s=test; t=1172620939; bh=TRy6u0KYixKENCPL95G1Y1uwclQ9rWthX6rFgE7gWb8=; h=Received:Received:Received:From:To:Date:Subject:Message-ID; b=aMDMwcz08Na0zYLrEZnUMBUipMh3blE+pWiBSvX+t2ynaDUGTqd27xuR5JddL9y1T6h0LkUMIcijsPBWa1WWfKL23QceksV75Y1/Tt6gmVi3SdVAoJbbZImrOaAQ3YQAn64dp1vI0Z8gSashYlwy+7gtmVSLAj/fpcAF6yNDPuE="
 #define SIG2 "v=1; a=rsa-sha256; c=relaxed/simple; d=example.com; s=test;\r\n\tt=1172620939; bh=TRy6u0KYixKENCPL95G1Y1uwclQ9rWthX6rFgE7gWb8=;\r\n\th=Received:Received:Received:From:To:Date:Subject:Message-ID; b=aMD\r\n\tMwcz08Na0zYLrEZnUMBUipMh3blE+pWiBSvX+t2ynaDUGTqd27xuR5JddL9y1T6h0Lk\r\n\tUMIcijsPBWa1WWfKL23QceksV75Y1/Tt6gmVi3SdVAoJbbZImrOaAQ3YQAn64dp1vI0\r\n\tZ8gSashYlwy+7gtmVSLAj/fpcAF6yNDPuE="
 
@@ -109,10 +111,10 @@ main(int argc, char **argv)
 
 	memset(body, '\0', sizeof body);
 
-	while (wlen > strlen(GIBBERISH))
+	while (wlen > GIBBERISHLEN)
 	{
 		sm_strlcat(body, GIBBERISH, sizeof body);
-		wlen -= strlen(GIBBERISH);
+		wlen -= GIBBERISHLEN;
 	}
 
 	status = dkim_body(dkim, body, strlen(body));
