@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char t_test73_c_id[] = "@(#)$Id: t-test73.c,v 1.4 2007/11/09 23:14:19 msk Exp $";
+static char t_test73_c_id[] = "@(#)$Id: t-test73.c,v 1.5 2007/12/12 00:16:25 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -164,7 +164,7 @@ main(int argc, char **argv)
 	assert(sig != NULL);
 
 	/* policy query, verify correct result */
-	status = dkim_policy(dkim, &test, &susp, &pcode, &hcode);
+	status = dkim_policy(dkim, &test, &susp, &pcode, &hcode, NULL);
 	assert(status == DKIM_STAT_OK);
 	assert(!test);
 	assert(susp);
