@@ -1,5 +1,5 @@
 /*
-**  Copyright (c) 2005, 2007 Sendmail, Inc. and its suppliers.
+**  Copyright (c) 2005, 2007, 2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 */
 
@@ -7,7 +7,7 @@
 #define _UTIL_H_
 
 #ifndef lint
-static char util_h_id[] = "@(#)$Id: util.h,v 1.12 2007/10/27 00:44:55 msk Exp $";
+static char util_h_id[] = "@(#)$Id: util.h,v 1.15 2008/01/23 03:11:49 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -23,8 +23,11 @@ extern int dkim_addrcmp __P((u_char *, u_char *));
 extern void dkim_collapse __P((u_char *));
 extern bool dkim_hdrlist __P((u_char *, size_t, u_char **, bool));
 extern int dkim_hexchar __P((int c));
-extern bool dkim_islwsp __P((u_int));
 extern void dkim_lowerhdr __P((u_char *));
 extern int dkim_qp_decode __P((u_char *, u_char *, int));
+extern unsigned long dkim_strtoul __P((const char *str, char **endptr,
+                                       int base));
+extern unsigned long long dkim_strtoull __P((const char *str, char **endptr,
+                                             int base));
 
 #endif /* ! _UTIL_H_ */
